@@ -31,10 +31,11 @@ TOther = TypeVar("TOther")
 class AsyncRx(AsyncObservable[TSource]):
     """An AsyncObservable class similar to classic Rx.
 
-    This class supports has all operators as methods and supports
+    This class provides all operators as methods and supports
     method chaining.
 
-    Subscribe is also a method.
+    Example:
+    >>> AsyncRx.from_iterable([1,2,3]).map(lambda x: x + 2).filter(lambda x: x < 3)
 
     All methods are lazy imported.
     """
@@ -112,6 +113,20 @@ class AsyncRx(AsyncObservable[TSource]):
     def from_iterable(cls, iter: Iterable[TSource]) -> "AsyncRx[TSource]":
         return AsyncRx(from_iterable(iter))
 
+    @classmethod
+    def from_async_iterable(cls, iter: AsyncIterable[TSource]) -> "AsyncObservable[TSource]":
+        """Convert an async iterable to an async observable stream.
+
+        Example:
+            >>> xs = AsyncRx.from_async_iterable(async_iterable)
+
+        Returns:
+            The source stream whose elements are pulled from the given
+            (async) iterable sequence.
+        """
+
+        return AsyncRx(from_async_iterable(iter))
+
     @classmethod
     def single(cls, value: TSource) -> "AsyncRx[TSource]":
         from .create import single
@@ -164,18 +179,16 @@ class AsyncRx(AsyncObservable[TSource]):
         return concat_seq([self, other])
 
     def debounce(self, seconds: float) -> "AsyncRx[TSource]":
-        """Debounce observable source.
+        """Debounce observable stream.
 
-        Ignores values from a source stream which are followed by
-        another value before seconds has elapsed.
+        Ignores values from an observable sequence which are followed by
+        another value before the given timeout.
 
-        Example:
-        partial = debounce(5) # 5 seconds
+        Args:
+            seconds (float): Number of seconds to debounce.
 
-        Keyword arguments:
-        seconds -- Duration of the throttle period for each value
-
-        Returns partially applied function that takes a source sequence.
+        Returns:
+            The debounced stream.
         """
 
         from .timeshift import debounce
@@ -252,11 +265,12 @@ class AsyncRx(AsyncObservable[TSource]):
         most-recently transformed observable sequence.
 
         Args:
-            mapper (Callable[[TSource]): [description]
-            Awaitable ([type]): [description]
+            mapper: Function to transform each item into a new async
+                observable.
 
         Returns:
-            Stream[TSource, TResult]: [description]
+            An async observable that only merges values from the latest
+            async observable produced by the mapper.
         """
         return AsyncRx(pipe(self, flat_map_latest_async(mapper)))
 
@@ -316,7 +330,8 @@ class AsyncRx(AsyncObservable[TSource]):
             count Number of elements to take.
 
         Returns:
-            Stream[TSource, TSource]: [description]
+            An observable sequence that contains the specified number of
+            elements from the start of the input sequence.
         """
         from .filtering import take
 
@@ -426,7 +441,7 @@ def debounce(seconds: float) -> Stream[TSource, TSource]:
     value before seconds has elapsed.
 
     Example:
-        >>> partial = debounce(5) # 5 seconds
+        >>> ys = pipe(xs, debounce(5)) # 5 seconds
 
     Args:
         seconds: Duration of the throttle period for each value
@@ -611,12 +626,19 @@ def flat_map_latest_async(mapper: Callable[[TSource], Awaitable[AsyncObservable[
     return flat_map_latest_async(mapper)
 
 
-def from_async_iterable(iter: Iterable[TSource]) -> "AsyncObservable[TSource]":
-    from aioreactive.operators.from_async_iterable import from_async_iterable
+def from_async_iterable(iter: AsyncIterable[TSource]) -> "AsyncObservable[TSource]":
+    """Convert an async iterable to an async observable stream.
 
-    from .create import from_async_iterable
+    Example:
+        >>> xs = rx.from_async_iterable(async_iterable)
 
-    return AsyncRx(from_async_iterable(iter))
+    Returns:
+        The source stream whose elements are pulled from the given
+        (async) iterable sequence.
+    """
+    from .create import of_async_iterable
+
+    return AsyncRx(of_async_iterable(iter))
 
 
 def interval(seconds: float, period: int) -> AsyncObservable[int]:
@@ -779,7 +801,8 @@ def take(count: int) -> Stream[TSource, TSource]:
         count Number of elements to take.
 
     Returns:
-        Stream[TSource, TSource]: [description]
+        An observable sequence that contains the specified number of
+        elements from the start of the input sequence.
     """
     from .filtering import take
 
