@@ -1,14 +1,24 @@
 import dataclasses
 import logging
 from dataclasses import dataclass
-from typing import Any, Callable, Generic, Iterable, NewType, Tuple, TypeVar, cast
+from typing import Any, Callable, Generic, Iterable, NoReturn, Tuple, TypeVar, cast
 
 from expression.collections import FrozenList, Map, frozenlist, map
-from expression.core import MailboxProcessor, Nothing, Option, Result, Some, TailCall, pipe, recursive_async
+from expression.core import (
+    MailboxProcessor,
+    Nothing,
+    Option,
+    TailCallResult,
+    Some,
+    TailCall,
+    match,
+    pipe,
+    tailrec_async,
+)
 from expression.system import AsyncDisposable
 
 from .create import of_seq
-from .msg import CompletedMsg, DisposeMsg, InnerCompletedMsg, InnerObservableMsg, Msg, OtherMsg, SourceMsg
+from .msg import CompletedMsg, DisposeMsg, InnerCompletedMsg, InnerObservableMsg, Key, Msg, OtherMsg, SourceMsg
 from .notification import Notification, OnNext
 from .observables import AsyncAnonymousObservable
 from .observers import AsyncAnonymousObserver, AsyncNotificationObserver, auto_detach_observer
@@ -19,8 +29,6 @@ TResult = TypeVar("TResult")
 TOther = TypeVar("TOther")
 
 
-Key = NewType("Key", int)
-
 log = logging.getLogger(__name__)
 
 
@@ -63,44 +71,44 @@ def merge_inner(max_concurrent: int = 0) -> Callable[[AsyncObservable[TSource]],
 
                 async def update(msg: Msg, model: Model[TSource]) -> Model[TSource]:
                     log.debug("update: %s, model: %s", msg, model)
-                    if isinstance(msg, InnerObservableMsg):
-                        msg = cast(InnerObservableMsg[TSource], msg)
-                        xs: AsyncObservable[TSource] = msg.inner_observable
-                        if max_concurrent == 0 or len(model.subscriptions) < max_concurrent:
-                            inner = await xs.subscribe_async(obv(model.key))
-                            return model.replace(
-                                subscriptions=model.subscriptions.add(model.key, inner),
-                                key=Key(model.key + 1),
-                            )
-                        else:
-                            return model.replace(queue=model.queue.append(frozenlist.singleton(xs)))
-                    elif isinstance(msg, InnerCompletedMsg):
-                        key = Key(msg.key)
-                        subscriptions = model.subscriptions.remove(key)
-                        if len(model.queue):
-                            xs = model.queue[0]
-                            inner = await xs.subscribe_async(obv(model.key))
+                    with match(msg) as m:
+                        for xs in InnerObservableMsg.case(m):
+                            if max_concurrent == 0 or len(model.subscriptions) < max_concurrent:
+                                inner = await xs.subscribe_async(obv(model.key))
+                                return model.replace(
+                                    subscriptions=model.subscriptions.add(model.key, inner),
+                                    key=Key(model.key + 1),
+                                )
+                            else:
+                                return model.replace(queue=model.queue.append(frozenlist.singleton(xs)))
+                        for key in InnerCompletedMsg.case(m):
+                            subscriptions = model.subscriptions.remove(key)
+                            if len(model.queue):
+                                xs = model.queue[0]
+                                inner = await xs.subscribe_async(obv(model.key))
 
-                            return model.replace(
-                                subscriptions=subscriptions.add(model.key, inner),
-                                key=model.key + 1,
-                                queue=model.queue.tail(),
-                            )
-                        elif len(subscriptions):
-                            return model.replace(subscriptions=subscriptions)
-                        else:
-                            if model.is_stopped:
+                                return model.replace(
+                                    subscriptions=subscriptions.add(model.key, inner),
+                                    key=model.key + 1,
+                                    queue=model.queue.tail(),
+                                )
+                            elif len(subscriptions):
+                                return model.replace(subscriptions=subscriptions)
+                            else:
+                                if model.is_stopped:
+                                    await safe_obv.aclose()
+                                return model.replace(subscriptions=map.empty)
+                        while CompletedMsg.case(m):
+                            if not model.subscriptions:
+                                log.debug("merge_inner: closing!")
                                 await safe_obv.aclose()
-                            return model.replace(subscriptions=map.empty)
-                    elif msg is CompletedMsg:
-                        if not model.subscriptions:
-                            log.debug("merge_inner: closing!")
-                            await safe_obv.aclose()
 
-                        return model.replace(is_stopped=True)
-                    else:
-                        for key, dispose in model.subscriptions:
-                            await dispose.dispose_async()
+                            return model.replace(is_stopped=True)
+
+                        while m.default():
+                            for key, dispose in model.subscriptions:
+                                await dispose.dispose_async()
+
                         return initial_model.replace(is_stopped=True)
 
                 async def message_loop(model: Model[TSource]) -> None:
@@ -170,10 +178,10 @@ def combine_latest(other: AsyncObservable[TOther]) -> Stream[TSource, Tuple[TSou
             safe_obv, auto_detach = auto_detach_observer(aobv)
 
             async def worker(inbox: MailboxProcessor[Msg]) -> None:
-                @recursive_async
+                @tailrec_async
                 async def message_loop(
                     source_value: Option[TSource], other_value: Option[TOther]
-                ) -> Result[AsyncObservable[TSource], Exception]:
+                ) -> TailCallResult[NoReturn]:
                     cn = await inbox.receive()
 
                     async def get_value(n: Notification[TSource]) -> Option[TSource]:
@@ -184,12 +192,14 @@ def combine_latest(other: AsyncObservable[TOther]) -> Stream[TSource, Tuple[TSou
                         await n.accept_observer(safe_obv)
                         return Nothing
 
-                    if isinstance(cn, SourceMsg):
-                        cn = cast(SourceMsg[TSource], cn)
-                        source_value = await get_value(cn.value)
-                    else:
-                        cn = cast(OtherMsg[TOther], cn)
-                        other_value = await get_value(cn.value)
+                    m = match(cn)
+                    for value in SourceMsg.case(m):
+                        source_value = await get_value(value)
+                        break
+
+                    for value in OtherMsg.case(m):
+                        other_value = await get_value(value)
+                        break
 
                     def binder(s: TSource) -> Option[Tuple[TSource, TOther]]:
                         def mapper(o: TOther) -> Tuple[TSource, TOther]:
@@ -245,8 +255,8 @@ def with_latest_from(other: AsyncObservable[TOther]) -> Stream[TSource, Tuple[TS
             safe_obv, auto_detach = auto_detach_observer(aobv)
 
             async def worker(inbox: MailboxProcessor[Msg]) -> None:
-                @recursive_async
-                async def message_loop(latest: Option[TOther]) -> Result[TSource, Exception]:
+                @tailrec_async
+                async def message_loop(latest: Option[TOther]) -> TailCallResult[NoReturn]:
                     cn = await inbox.receive()
 
                     async def get_value(n: Notification[TSource]) -> Option[TSource]:
